﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.Serialization;
using System.Data.OleDb;

namespace Clients
{
    public partial class Form1 : Form
    {
        private config Config;
        private string connectionString;
        private bool lFilter;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            if (File.Exists("config.xml"))
            {
                try
                {
                    // Если файл настроек обнаружен пытаемся с ним работать через сериалайзер
                    Config = new config();
                    XmlSerializer serializer = new XmlSerializer(typeof(config));
                    TextReader reader = new StreamReader("config.xml");
                    Config = (config)serializer.Deserialize(reader);
                    reader.Close();
                    // из считанных настроек формируем строку соединения с БД
                    connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + Config.DBPath;
                    

                }
                catch
                {
                    return;
                }
            }
            else
            {
                // если файла настроек нет, то формируем парамерты по умолчанию
                // и записываем их в файл настроек для дальнейшего использования
                // запись через сериалайзер
                Config = new config();
                Config.DBPath = "clients.mdb";
                Config.DBAdmin = "Admin";
                XmlSerializer serializer = new XmlSerializer(typeof(config));
                TextWriter writer = new StreamWriter("config.xml");
                serializer.Serialize(writer, Config);
                writer.Close();
                connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + Config.DBPath;
            }
            // Загрузка таблицы
            // перед загрузкой сбрасываем признак установленного фильтра
            lFilter = false;
            this.reloadTable();
        }

        private void reloadTable()
        {
            // чистим таблицу
            this.dataGridView1.Rows.Clear();
            // созадем подключение и открываем его
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            // строка выбора всех полей из таблицы
            string command = "select * from [ClientsList]";
            if (lFilter)
            {
                // если взведен признак фильтра, то формируем ту часть запроса
                // которая отвечает за условие отбора
                // при этом используется команда LIKE, которая позволяет делать сравнение на вхождение по маске
                string cName, cPhone, cAddress;
                bool lNoWhere = false;
                cName = this.textBox3.Text;
                cPhone = this.textBox5.Text;
                cAddress = this.textBox4.Text;
                if (cName.Length > 0)
                {
                    lNoWhere = true;
                    // на всякий случай заменяем одинарные кавычки на двойные, а знак процента на двойной процент - это необходимо чтобы не завалить БД
                    // ЗЫ: это один из способов ломануть сайт, построенный на субд, если программист забыл это предусмотреть
                    command += " where [ClientName] like '%" + cName.Replace("'", "\"").Replace("%", "%%") + "%'";
                }

                if (cPhone.Length > 0)
                {

                    if (!lNoWhere)
                    {
                        command += " where ";
                        lNoWhere = true;
                    }
                    else
                        command += " and ";

                    command += "[ClientTelNumber] like '%" + cPhone.Replace("'", "\"").Replace("%", "%%") + "%'";
                }

                if (cAddress.Length > 0)
                {
                    if (!lNoWhere)
                    {
                        command += " where ";
                        lNoWhere = true;
                    }
                    else
                        command += " and ";

                    command += "[ClientAddress] like '%" + cAddress.Replace("'", "\"").Replace("%", "%%") + "%'";
                }
            }
            // создаем класс запроса
            OleDbCommand cmd = new OleDbCommand(command, conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nRow;
            // пока таблица дает строки работает цикл
            while (myOleDbDataReader.Read())
            {
                // добавляем строку и заполняем колонки
                nRow = this.dataGridView1.Rows.Add();
                this.dataGridView1.Rows[nRow].Cells["ColumnID"].Value = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                this.dataGridView1.Rows[nRow].Cells["ColumnName"].Value = myOleDbDataReader["ClientName"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnAddress"].Value = myOleDbDataReader["ClientAddress"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnPhone"].Value = myOleDbDataReader["ClientTelNumber"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnComment"].Value = myOleDbDataReader["ClientComments"].ToString();
            }
            // закрываем открытые подключения
            myOleDbDataReader.Close();
            conn.Close();
            // включаем или выключаем кнопки на панели инструментов
            bindingNavigatorAddNewItem.Enabled = true;
            if (dataGridView1.Rows.Count > 0)
            {
                bindingNavigatorMoveFirstItem.Enabled = true;
                bindingNavigatorMovePreviousItem.Enabled = true;
                bindingNavigatorMoveNextItem.Enabled = true;
                bindingNavigatorMoveLastItem.Enabled = true;
                bindingNavigatorDeleteItem.Enabled = true;
                dataGridView1.Rows[0].Selected = true;
            }
            else
            {
                bindingNavigatorMoveFirstItem.Enabled = false;
                bindingNavigatorMovePreviousItem.Enabled = false;
                bindingNavigatorMoveNextItem.Enabled = false;
                bindingNavigatorMoveLastItem.Enabled = false;
                bindingNavigatorDeleteItem.Enabled = false;
            }
        }


        private void bindingNavigatorMoveFirstItem_Click(object sender, EventArgs e)
        {
            // переход на первую строку таблицы
            dataGridView1.CurrentCell = dataGridView1.Rows[0].Cells[dataGridView1.CurrentCell.ColumnIndex];
        }

        private void bindingNavigatorMovePreviousItem_Click(object sender, EventArgs e)
        {
            // переход на предыдущую строку таблицы
            if (dataGridView1.CurrentRow.Index > 0)
            {
                dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.CurrentRow.Index - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
            }
        }

        private void bindingNavigatorMoveNextItem_Click(object sender, EventArgs e)
        {
            // переход на следующую строку таблицы
            if (dataGridView1.CurrentRow.Index < dataGridView1.Rows.Count-1)
            {
                dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.CurrentRow.Index + 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
            }
        }

        private void bindingNavigatorMoveLastItem_Click(object sender, EventArgs e)
        {
            // переход на последнюю строку таблицы
            dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
        }

        private void bindingNavigatorAddNewItem_Click(object sender, EventArgs e)
        {
            // добавление новой записи
            // открываем подключение
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            // получаем максимальную код идентификатора
            string command = "select Max(ClientID) from [ClientsList]";
            OleDbCommand cmd = new OleDbCommand(command, conn);
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nID;
            // если таблица не пустая, то берем максимум и наращиваем на один
            if (myOleDbDataReader.Read())
                nID = int.Parse(myOleDbDataReader[0].ToString())+1;
            // если таблица пустая, то первая запись
            else
                nID = 1;
            // закрываем табличку
            myOleDbDataReader.Close();
            // делаем новую запись с заполнением идентификатора
            cmd.CommandText = "INSERT INTO [ClientsList] ([ClientID]) VALUES (" + nID.ToString() + ")";
            if (cmd.ExecuteNonQuery() > 0)
            {
                // если все создалось, то делаем реинициализацию таблицы и заходим на последнюю строку таблицы (это вновь созданная строка)
                conn.Close();
                lFilter = false;
                button4.Enabled = false;
                this.reloadTable();
                this.dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
                // здесь выходим из метода, т.к. если этого не сделать - следующая строка закрытия выпадет
                return;
            }
            conn.Close();
        }

        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            // удаление записи
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            string idc = this.dataGridView1.CurrentRow.Cells[0].Value.ToString();
            string command = "delete from [ClientsList] where [ClientID] = "+idc;
            OleDbCommand cmd = new OleDbCommand(command, conn);
            if (cmd.ExecuteNonQuery() > 0)
            {
                // если команда отработала, то удаляем из таблицы текущую запись
                dataGridView1.Rows.Remove(dataGridView1.CurrentRow);
            }
            conn.Close();
            // если удалена последняя запись, то нужно выключить кнопки на панели инструментов
            if (dataGridView1.Rows.Count == 0)
            {
                bindingNavigatorMoveFirstItem.Enabled = false;
                bindingNavigatorMovePreviousItem.Enabled = false;
                bindingNavigatorMoveNextItem.Enabled = false;
                bindingNavigatorMoveLastItem.Enabled = false;
                bindingNavigatorDeleteItem.Enabled = false;
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            // выключаем фильтр
            lFilter = false;
            this.button4.Enabled = false;
            this.button3.Enabled = true;
            this.reloadTable();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            // включаем фильтр
            lFilter = true;
            this.button4.Enabled = true;
            this.button3.Enabled = false;
            this.reloadTable();
        }

        private void dataGridView1_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // событие завершения редактирования поля в таблице
            // получаем код клиента
            string ID = this.dataGridView1.Rows[e.RowIndex].Cells[0].Value.ToString();
            string namefld = "";
            // получаем новый текст, введенный пользователем
            string newdata = this.dataGridView1.Rows[e.RowIndex].Cells[e.ColumnIndex].Value.ToString();
            // определяем какое поле менялось
            switch (e.ColumnIndex)
            {
                case 1:
                    namefld = "ClientName";
                    break;
                case 2:
                    namefld = "ClientAddress";
                    break;
                case 3:
                    namefld = "ClientTelNumber";
                    break;
                case 4:
                    namefld = "ClientComments";
                    break;
            }
            // если не смогли определить, то выходим
            if (namefld.Length == 0)
                return;
            OleDbConnection conn = new OleDbConnection(connectionString);
            conn.Open();
            // формируем запрос обновления данных в таблице БД
            OleDbCommand cmd = new OleDbCommand("UPDATE ClientsList SET ["+namefld+"] = '" + newdata + "' where [ClientId] = " + ID, conn);
            cmd.ExecuteNonQuery();
            conn.Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // поиск по имени
            string name = this.textBox1.Text;
            if (name.Length > 0)
            {
                OleDbConnection conn = new OleDbConnection(connectionString);
                conn.Open();
                // делаем запрос на получение идентификатора по первому совпадению текста в подстроке
                OleDbCommand cmd = new OleDbCommand("select [ClientID] from ClientsList where [ClientName] like '%" + name + "%'", conn);
                OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    // если нашли, то берем идентификатор 
                    string id = myOleDbDataReader[0].ToString();
                    // и перебирая все записи находим нужную
                    foreach(DataGridViewRow row in dataGridView1.Rows)
                    {
                        if (row.Cells[0].Value.ToString() == id)
                        {
                            // если нашли, то делаем ее текущей и выходим из цикла (зачем лишние движения?)
                            dataGridView1.CurrentCell = row.Cells[0];
                            break;
                        }
                    }
                }
                myOleDbDataReader.Close();
                conn.Close();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            // поиск по телеофну (описание см. выше)
            string name = this.textBox2.Text;
            if (name.Length > 0)
            {
                OleDbConnection conn = new OleDbConnection(connectionString);
                conn.Open();
                OleDbCommand cmd = new OleDbCommand("select [ClientID] from ClientsList where [ClientTelNumber] like '%" + name + "%'", conn);
                OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    string id = myOleDbDataReader[0].ToString();
                    foreach (DataGridViewRow row in dataGridView1.Rows)
                    {
                        if (row.Cells[0].Value.ToString() == id)
                        {
                            dataGridView1.CurrentCell = row.Cells[0];
                            break;
                        }
                    }
                }
                myOleDbDataReader.Close();
                conn.Close();
            }
        }
    }
}
