using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Guardant;
using System.Security.Cryptography; // Используем Microsoft CryptoAPI


namespace GenerateAES1000
{

    class Program
    {
        static int Main(string[] args)
        {
            #region Variables definition
            // ------------------------------------------------
            // Variables definition
            // ------------------------------------------------

            GrdE RetCode;                       // Код ошибки для всех вызовов Guardant API

            Handle GrdHandle = new Handle();    // Создает пустой хэндл контейнера Guardant

            // Константы
            uint CryptPU = 0x8566783U;         // Случайное значение для скрытия кодов доступа                              
            uint CryptRD = 0x17d49c84U;        // Случайное значение для скрытия кодов доступа                             

            // Демо коды доступа увеличиваются скрываются методом сложения, чтобы их нельзя было найти
            // в коде программы в чистом виде
            uint PublicCode = (uint)GrdDC.DEMONVK - CryptPU;    // Должно быть расшифровано            
            uint ReadCode = (uint)GrdDC.DEMORDO - CryptRD;    // Должно быть расшифровано            

            // Переменные для метода GrdSetFindMode()
            GrdFMR RemoteMode;         // Флаги поиска локальных или сетевых ключей                  
            GrdFM DongleFlags;        // Флаги поиска ключа                  
            uint ProgramNumber = 0;  // Номер программы                      
            uint Version = 0;        // Версия программы                                 
            uint DongleID = 0;       // ID ключа (Номер ключа)
            byte[] TypeID = new byte[2];// Тип ключа
            uint SerialNumber;       // Сериный номер                           
            uint BitMask;            // Битовая маска                           
            GrdDT DongleType;         // Тип ключа                            
            GrdFMM DongleModel;        // Модель ключа                           
            GrdFMI DongleInterface;    // Тип интерфейса ключа
            byte[] InitVectorAES;       // Переменная вектора инициализации для алгоритма AES
            byte[] DataString = new byte[16]; // Массив данных для шифрования
            
            byte[] DataPassword = new byte[32];
            byte[] GrdAES_Context = new byte[(int)GrdAES.CONTEXT_SIZE];

            // Криптопровайдер для инициализации функций Microsoft CryptoAPI
            RNGCryptoServiceProvider rngCsp;

            #endregion

            // Отображаем сообщение о старте
            System.Console.WriteLine("START AES generation");

            // Инициализация криптопровайдера Microsoft CryptoAPI
            rngCsp = new RNGCryptoServiceProvider();

            // Инициализация копии GrdAPI. GrdStartup() должен быть вызван перед первым вызовом функций GrdAPI 
            RetCode = GrdApi.GrdStartup(GrdFMR.Local);	// Мы будем использовать только локальные ключи
            ErrorHandling(GrdHandle, RetCode);

            // -----------------------------------------------------------------
            // Создание контейнера Guardant и получение его хэндла
            // -----------------------------------------------------------------
            GrdHandle = GrdApi.GrdCreateHandle(GrdCHM.MultiThread);
            if (GrdHandle.Address == 0) // Если найдена ошибка
            {
                ErrorHandling(new Handle(0), GrdE.MemoryAllocation);
            }

            // -----------------------------------------------------------------
            // Установка кодов доступа к ключу защиты Guardant
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdSetAccessCodes(GrdHandle,	// хэндл контейнера Guardant 
                PublicCode + CryptPU,   // Публичный код. Используется для поиска ключа. Должен быть установлен
                ReadCode + CryptRD,     // Приватный код для чтения; Данный код используется для чтения незащищенных ячеек памяти;
                0,	// Приватный код для записи. В большинстве случаев, он не нужен. Старайтесь не включать его в итоговые версии приложения.
                0);  // Мастер код. Используется для редактирования параметров ключа. Никогда не вставляйте этот код в публичные версии приложения!

            ErrorHandling(GrdHandle, RetCode); // Обработка ошибок


            // -----------------------------------------------------------------
            // Устанавливаем параметры поиска ключа
            // -----------------------------------------------------------------
            RemoteMode = GrdFMR.Local; 				            // Поиск только локальных ключей
            DongleFlags = GrdFM.NProg & GrdFM.Mask;	            // Устанавливаем флаг поиска по номеру программы и маске
            ProgramNumber = 5;     								// Устанавливаем номер программы. В нашем поиске не используется.                
            DongleID = 0;       		     					// ID ключа. Не используется
            SerialNumber = 0;     							    // Серийный номер. В нашем поиске не используется.                     
            Version = 0;    									// Версия. В нашем поиске не используется                   
            BitMask = 77;     									// Битовая маска. В нашем поиске не используется.                    
            DongleType = GrdDT.GSII64;				            // Ключ с поддержкой алгоритма GSII64 
            DongleModel = GrdFMM.ALL;					        // Все модели ключей. 
            DongleInterface = GrdFMI.ALL;		   		        // Интерфейс подключения любой (LPT или USB)

            // Устанавливаем параметры поиска ключей защиты.
            // Все последующие запуски функций GrdFind() и GrdLogin() перед следующий изменением параметров, функцией 
            // GrdSetFindMode() будут использовать установленные критерии поиска.
            // Если значения полей ключа и заданные не совпадут, то выдастся ошибка.
            RetCode = GrdApi.GrdSetFindMode(GrdHandle,
                                            RemoteMode,
                                            DongleFlags,
                                            ProgramNumber,
                                            DongleID,
                                            SerialNumber,
                                            Version,
                                            BitMask,
                                            DongleType,
                                            DongleModel,
                                            DongleInterface);
            ErrorHandling(GrdHandle, RetCode);

            // Производим подключение к ключу (с параметрами поиска заданными GrdFindMode).
            // Все последующие вызовы функций Guardant API, до следующего вызова GrdCloseHandle()/GrdLogin()
            // будут использовать подключенный ключ
            RetCode = GrdApi.GrdLogin(GrdHandle, 0xFFFFFFFF, GrdLM.PerStation);
            ErrorHandling(GrdHandle, RetCode);
 
	        // Активируем алгоритм для работы
            RetCode = GrdApi.GrdPI_Activate (GrdHandle,	// Хэндл контейнера Guardant
		                    05,					            // Номер защищенной ячейки или алгоритма
		                    875679933 );	                // Пароль для активации ячейки или алгоритма 
            ErrorHandling(GrdHandle, RetCode);
            

            //Символ @ блокирует использование косой черты как маркера начала спецсимвола
            string path = "result_1000.txt";

            //Создаем файл и объект streamwriter, который будет
                //существовать до закрытия фигурной скобки
                using (StreamWriter streamwriter = File.CreateText(path))
                {
                    streamwriter.WriteLine("byte[][] aes1000source = new byte[1000][];");
                    streamwriter.WriteLine("byte[][] aes1000crypt = new byte[1000][];");
                    streamwriter.WriteLine("");
                    for (int i1000 = 0; i1000 < 1000; i1000++)
                    {
                        // Заполняем массив случайными числами
                        rngCsp.GetBytes(DataString);

                        streamwriter.WriteLine("aes1000source[" + Convert.ToString(i1000) + "] = new byte[16] " + ByteArrayToHexString(DataString) + ";");


                        // Вектор инициализации алгоритма AES 
                        InitVectorAES = new byte[] {14, 215, 77, 46, 93, 121, 115, 253, 28, 91, 28, 139, 214, 203, 135, 93 };

                         // Шифруем случайную строку на однонаправленном алгоритме AES 
                        RetCode = GrdApi.GrdCrypt(GrdHandle,	// Хэндл контейнера Guardant
                                    05,				            // Номер аппаратного алгоритма
                                    DataString,					// Данные для шифрования
                                    (UInt32)GrdAM.Encrypt +
                                    (UInt32)GrdAM.CBC, 			// Шифровать все блоки по методу CBC
                                    InitVectorAES,			    // Вектор инициализации
                                    DataPassword,				// Только для программно-реализованных алгоритмов
                                    GrdAES_Context);			// Только для программно-реализованных алгоритмов

                        ErrorHandling(GrdHandle, RetCode);

                        streamwriter.WriteLine("aes1000crypt[" + Convert.ToString(i1000) + "] = new byte[16] " + ByteArrayToHexString(DataString) + ";");


                    } // конец цикла 1000

                } // Завершение записи в файл


            // Деактивируем алгоритм
            RetCode = GrdApi.GrdPI_Deactivate (GrdHandle,	// Хэндл контейнера Guardant
		                    05,					            // Номер защищенной ячейки или алгоритма
		                    136056856 );	                // Пароль для деактивации ячейки или алгоритма 
            ErrorHandling(GrdHandle, RetCode);

            // Завершаем сеанс работы с ключом
            RetCode = GrdApi.GrdLogout(GrdHandle);
            ErrorHandling(GrdHandle, RetCode);

            // Закрываем хэндл контейнера Guardant
            RetCode = GrdApi.GrdCloseHandle(GrdHandle);
            ErrorHandling(GrdHandle, RetCode);

            // Деинициализация копии Guardant API. Метод GrdCleanup() должен быть вызван перед закрытием программы
            RetCode = GrdApi.GrdCleanup();
            ErrorHandling(GrdHandle, RetCode);

            System.Console.Write("\nEND AES generation. Press Enter to Exit ");

            Console.ReadLine();
            return (int)GrdE.OK;
        }

          //----------------------------------------------------------------------------------------------
        //  Функция обработки ошибок
        //  Показывает результат, закрывает хэндл, и завершает работу программы
        //  Вход:  Хэндл контейнера Guardant 
        //	Вход :  код ошибки
        //	Выход: код ошибки
        //----------------------------------------------------------------------------------------------
        private static GrdE ErrorHandling(Handle hGrd, GrdE nRet)
        {
            if (nRet != GrdE.OK)
            {
                // Выводим ошибку последней функции

            System.Console.Write("\nError: "+GrdApi.PrintResult((int)nRet));

                if (hGrd.Address != 0)	// Проверяем существует ли хэндл контейнера Guardant 
                {
                    // Закрываем хэндл, выходим с сервера ключа,  освобождаем память
                    nRet = GrdApi.GrdCloseHandle(hGrd);
                }

                // Деинициализация копии Guardant API. Метод GrdCleanup() должен быть вызван перед закрытием программы
                nRet = GrdApi.GrdCleanup();
                Console.ReadLine();
                // Terminate application
                Environment.Exit((int)nRet);
            }
            return nRet;
        }
        
        static public string ByteArrayToHexString(byte[] Bytes)  
        {  
            StringBuilder Result = new StringBuilder();  
            string HexAlphabet = "0123456789ABCDEF";
            Result.Append("{");
            foreach (byte B in Bytes)  
            {
                Result.Append("0x");
                Result.Append(HexAlphabet[(int)(B >> 4)]);  
                Result.Append(HexAlphabet[(int)(B & 0xF)]);
                Result.Append(", ");
            }
            Result.Remove(Result.Length-2,2);
            Result.Append("}");

            return Result.ToString();  
        }  

    }
}
